/* Copyright 1998-2012 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_BASELINE_NODES_H
#define QUB_BASELINE_NODES_H

typedef struct QUB_BaselineSeg_Node {
	int point;
	float value;
} QUB_BaselineSeg_Node;

typedef struct QUB_BaselineSeg{
	int first;
	int last;
} QUB_BaselineSeg;

#ifdef __cplusplus
extern "C" {
#endif


  QUBFAST_API QUB_BaselineSeg *       QUB_BaselineSeg_Create( int first, int last );
  QUBFAST_API void                    QUB_BaselineSeg_Free( QUB_BaselineSeg *seg );
  QUBFAST_API int                     QUB_BaselineSeg_GetNodeCount( QUB_BaselineSeg *seg );
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_GetFirstNode( QUB_BaselineSeg *seg );
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_GetLastNode( QUB_BaselineSeg *seg );
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_GetNodeBefore( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node *node );
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_GetNodeAfter( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node *node );

  QUBFAST_API float QUB_BaselineSeg_InterpolateValueAt( QUB_BaselineSeg *seg, int point );
// next three: return non-zero (negative) for error e.g. out-of-range / past-the-end
  QUBFAST_API int   QUB_BaselineSeg_GetRangeAt( QUB_BaselineSeg *seg, int point, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );
  QUBFAST_API int   QUB_BaselineSeg_GetRangeBefore( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );
  QUBFAST_API int   QUB_BaselineSeg_GetRangeAfter( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );

// existing nodes at the point / in the range given will be cleared first
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_AddNode( QUB_BaselineSeg *seg, int point, float value );
  QUBFAST_API QUB_BaselineSeg_Node *  QUB_BaselineSeg_AddNodes( QUB_BaselineSeg *seg, int count, int *points, float *values );
  QUBFAST_API void                    QUB_BaselineSeg_ClearNodes( QUB_BaselineSeg *seg, int first, int last, int repairFirstLast );

#ifdef __cplusplus
}
#endif

#endif
